package com.jpa;

import java.util.Collection;
import java.util.HashSet;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;

import org.hibernate.annotations.BatchSize;

@Entity
@NamedQueries({
	@NamedQuery(
		name = "Person.findByName", 
		query = "SELECT p FROM Person p WHERE p.name = :name"),
	@NamedQuery(
			name = "Person.findByName.fetchOrgs", 
			query = "SELECT DISTINCT p FROM Person p LEFT JOIN FETCH p.organisations WHERE p.name = :name"),
})
public class Person {

	public static final String TABLE_CREATE = "create table PERSON (id int not null, name varchar(32))";

	@Id
	@GeneratedValue
	private long id;

	private String name;
	
	@OneToMany(mappedBy="person", cascade=CascadeType.ALL, fetch=FetchType.LAZY)
	@BatchSize(size=3)
	private Collection<Organisation> organisations = new HashSet<Organisation>();

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void addOrganisation(Organisation organisation) {
		organisation.setPerson(this);
		this.organisations.add(organisation);
	}

	public Collection<Organisation> getOrganisations() {
		return organisations;
	}
}