package com.jpa.test;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.jpa.Organisation;
import com.jpa.Person;

public class PersonTests extends TestCase {

    protected EntityManager em = TestUtils.EMF.createEntityManager();
	
    public PersonTests(String name) {
    	super(name);
    }

	@Override
	protected void setUp() throws Exception {
	    em.clear();
	}

	@Override
	protected void tearDown() throws Exception {
	}
	
	public void testPersonCreate() {
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Person person = TestUtils.createPerson();
	    em.persist(person);
	    userTransaction.commit();
	}
	
	public void testPersonFindByID() {
		// erstelle eine Person
		EntityTransaction userTransaction = em.getTransaction();
	    userTransaction.begin();
	    Person person = TestUtils.createPerson();
	    em.persist(person);
	    userTransaction.commit();
	    // merke die Inhalte:
	    long personID = person.getId();
	    String personName = person.getName();
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Person personFound = em.find(Person.class, personID);
	    
	    Assert.assertEquals("Name der Person muss gleich sein.", personName, personFound.getName());
	}

	public void testPersonFindWithQuery() {
		// erstelle eine Person
		// erstelle eine 10 Personen mit 10 Orgs und fange sie alle mit einem Select

		String personName = "C" + TestUtils.randomString(); 
	    for (int j = 0; j < 10; j++) {
	    	EntityTransaction userTransaction = em.getTransaction();
	    	userTransaction.begin();
	    	Person person = TestUtils.createPerson();
	    	person.setName(personName);
		    for (int i = 0; i < 10; i++) {
		    	person.addOrganisation(TestUtils.createOrganisation());
		    };
		    em.persist(person);
		    userTransaction.commit();
	    };
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Query query = em.createNamedQuery("Person.findByName");
	    query.setParameter("name", personName);
	    List<Person> persons = query.getResultList();
	    Assert.assertEquals("Es werden 10 Personen werwartet.", 10, persons.size());
	    
	    // gehe durch alle Personen 
	    for (Person person : persons) {
	    	System.out.println("--Person read-- " + person.getId());
	    	Collection<Organisation> organisations = person.getOrganisations();
	    	for (Organisation organisation : organisations) {
		    	System.out.println("   --Org read--" + organisation.getId());
	    	}
		}
	}
	
	public void testPersonWithOrganisation() {
		// erstelle eine Person
	  
	    Person person = TestUtils.createPerson();
	    for (int j = 0; j < 10; j++) {
			EntityTransaction userTransaction = em.getTransaction();
		    userTransaction.begin();
		    for (int i = 0; i < 10; i++) {
		    	person.addOrganisation(TestUtils.createOrganisation());
		    };
		    em.persist(person);
		    userTransaction.commit();
	    };
	    // merke die Inhalte:
	    long personID = person.getId();
	    String personName = person.getName();
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Person personFound = em.find(Person.class, personID);
	    Assert.assertEquals("Name der Person muss gleich sein.", personName, personFound.getName());
    	System.out.println("\n-----");
	    
	    // Lese die Organisationen
    	int counter = 0;
    	System.out.println("\nFound: " + personFound.getOrganisations().size());
    	for (Iterator<Organisation> iterator = personFound.getOrganisations().iterator(); iterator.hasNext();) {
			Organisation orgfound = iterator.next();
	    	orgfound.getName();
	    	counter++;
	    }
    	System.out.println("\nFound: " + counter + " Organisations");
	}
	
	public void testPersonenWithOrganisationJoinFetch() {
		// erstelle eine 10 Personen mit 10 Orgs und fange sie alle mit einem Select

		String personName = "C" + TestUtils.randomString(); 
	    for (int j = 0; j < 10; j++) {
	    	EntityTransaction userTransaction = em.getTransaction();
	    	userTransaction.begin();
	    	Person person = TestUtils.createPerson();
	    	person.setName(personName);
		    for (int i = 0; i < 10; i++) {
		    	person.addOrganisation(TestUtils.createOrganisation());
		    };
		    em.persist(person);
		    userTransaction.commit();
	    };
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut
	    Query query = em.createNamedQuery("Person.findByName.fetchOrgs");
	    query.setParameter("name", personName);
	    List<Person> persons = query.getResultList();
	    Assert.assertEquals("Es werden 10 Personen werwartet.", 10, persons.size());
	    
	    // gehe durch alle Personen 
	    for (Person person : persons) {
	    	System.out.println("--Person read-- " + person.getId());
	    	Collection<Organisation> organisations = person.getOrganisations();
	    	for (Organisation organisation : organisations) {
		    	System.out.println("   --Org read--" + organisation.getId());
	    	}
		}
	}
	
	public void testPersonenWithOrganisationJoinFetchAndPaging() {
		// erstelle eine 10 Personen mit 10 Orgs und fange sie alle mit einem Select

		String personName = "C" + TestUtils.randomString(); 
	    for (int j = 0; j < 10; j++) {
	    	EntityTransaction userTransaction = em.getTransaction();
	    	userTransaction.begin();
	    	Person person = TestUtils.createPerson();
	    	person.setName(personName);
		    for (int i = 0; i < 10; i++) {
		    	person.addOrganisation(TestUtils.createOrganisation());
		    };
		    em.persist(person);
		    userTransaction.commit();
	    };
	    // das große Vergessen:
	    em.clear();
	    
	    // finde Person erneut, diesmal Paging:
	    final int pageSize = 5;
	    for (int page = 0; page < 2; page++) {
	    	System.out.println("--Page-- " + page);
	    	
		    Query query = em.createNamedQuery("Person.findByName.fetchOrgs");
		    query.setParameter("name", personName);
		    query.setFirstResult(pageSize * page);
		    query.setMaxResults(pageSize);
		    
		    List<Person> persons = query.getResultList();
		    Assert.assertEquals("Es werden " + pageSize +" Personen werwartet.", pageSize, persons.size());
		    
		    // gehe durch alle Personen 
		    for (Person person : persons) {
		    	System.out.println("--Person read-- " + person.getId());
		    	Collection<Organisation> organisations = person.getOrganisations();
		    	for (Organisation organisation : organisations) {
			    	System.out.println("   --Org read--" + organisation.getId());
		    	}
			}
	    }
	}
}