package com.jpa.test;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import junit.framework.TestCase;

import com.jpa.Organisation;
import com.jpa.Person;

public class WorkWithTablesTest extends TestCase {

	private Connection connection;

	@Override
	protected void setUp() throws Exception {
		Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
		Properties properties = new Properties();
		properties.put("user", "user1");
		properties.put("password", "user1");

		this.connection = DriverManager.getConnection(
				"jdbc:derby:c:/TEMP/testDB;create=true", properties);
	}

	@Override
	protected void tearDown() throws Exception {
		this.connection.close();
	}
	 
	public void rebuildTables() throws SQLException {
		dropTables();
		createTableTestIfNotExistingYet();
	}

	public void createTableTestIfNotExistingYet()
			throws SQLException {
		// look in the meta info
		ResultSet resultSet = connection.getMetaData().getTables("%", "%", "%",
				new String[] { "TABLE" });
		boolean foundOpenJPASequence = false;
		boolean foundPerson = false;
		boolean foundOrganisation = false;
		//boolean foundHibernateUniqueKey = false;
		while (resultSet.next()) {
			String name = resultSet.getString("TABLE_NAME");
			System.out.println("found Table: " + name);
			foundOpenJPASequence |= "OPENJPA_SEQUENCE_TABLE".equalsIgnoreCase(name);
			foundPerson |= "PERSON".equalsIgnoreCase(name);
			foundOrganisation |= "ORGANISATION".equalsIgnoreCase(name);
		}
		if (!foundOpenJPASequence) {
			System.out.println("Creating Table OPENJPA_SEQUENCE_TABLE...");
			Statement statement = connection.createStatement();
			statement.execute("create table OPENJPA_SEQUENCE_TABLE (ID int not null, SEQUENCE_VALUE varchar(32))");
			statement.close();
		}
		if (!foundPerson) {
			System.out.println("Creating Table PERSON...");
			Statement statement = connection.createStatement();
			statement.execute(Person.TABLE_CREATE);
			statement.close();
		}
		if (!foundOrganisation) {
			System.out.println("Creating Table ORGANISATION...");
			Statement statement = connection.createStatement();
			statement.execute(Organisation.TABLE_CREATE);
			statement.close();
		}
		// ...
	}

	public void dropTables() throws SQLException {
		ResultSet resultSet = connection.getMetaData().getTables("%", "%", "%",
				new String[] { "TABLE" });
		while (resultSet.next()) {
			String name = resultSet.getString("TABLE_NAME");
			System.out.println("about to drop table " + name);
			Statement statement = connection.createStatement();
			statement.execute("DROP TABLE " + name);
			statement.close();
		}
	}

	public void testShowPerson() throws SQLException {
		showContentsOfTable(this.connection, "PERSON");
	}

	public void testShowOrganisation() throws SQLException {
		showContentsOfTable(this.connection, "ORGANISATION");
	}

	public void testShowSequenceTable() throws SQLException {
		showContentsOfTable(this.connection, "OPENJPA_SEQUENCE_TABLE");
	}

	private void showContentsOfTable(Connection connection, String tableName)
			throws SQLException {

		Statement statement = connection.createStatement();
		ResultSet resultSet = statement.executeQuery("SELECT * FROM "
				+ tableName);
		int columnCnt = resultSet.getMetaData().getColumnCount();
		for (int i = 1; i <= columnCnt; i++) {
			System.out.print(resultSet.getMetaData().getColumnName(i) + " ");
		}
		System.out.println();
		System.out.println("----------------------------------------");
		while (resultSet.next()) {
			for (int i = 1; i <= columnCnt; i++) {
				System.out.print("'");
				System.out.print(resultSet.getString(i));
				System.out.print("' ");
			}
			System.out.println();
		}
		resultSet.close();
		statement.close();
	}
}
